{% spaceless %}{% load i18n %}{% autoescape off %}{% blocktrans %}
You are receiving this email because you requested a password reset
for your user account at {{ site.name }}.

Please go to the following page and choose a new password:{% endblocktrans %}
{{ protocol }}://{{ site.domain }}{% url 'user_management_api:password_reset_confirm' uidb64=uid token=token %}
{% blocktrans %}
The {{ site.name }} team.
{% endblocktrans %}{% endautoescape %}
{% endspaceless %}
